Package: ADAPT
Title: Analysis of Microbiome Differential Abundance by Pooling Tobit
        Models
Version: 1.6.0
Authors@R: 
    c(person(given="Mukai", family="Wang", email="wangmk@umich.edu", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-1413-1904")),
      person(given="Simon", family="Fontaine", email="simfont@umich.edu", role = "ctb"),
      person(given="Hui", family="Jiang", email="jianghui@umich.edu", role="ctb"),
      person(given="Gen", family="Li", email="ligen@umich.edu", role=c("aut", "ctb")))
Description: ADAPT carries out differential abundance analysis for microbiome metagenomics data in phyloseq format.
  It has two innovations. One is to treat zero counts as left censored and use Tobit models for log count ratios.
  The other is an innovative way to find non-differentially abundant taxa as reference, then use the reference taxa to 
  find the differentially abundant ones.
License: MIT + file LICENSE
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.2
Imports: Rcpp (>= 1.0.8), RcppArmadillo (>= 0.10.8), RcppParallel (>=
        5.1.5), phyloseq (>= 1.39.0), methods, stats, ggplot2 (>=
        3.4.1), ggrepel (>= 0.9.1)
Suggests: rmarkdown (>= 2.11), knitr (>= 1.37), testthat (>= 3.0.0)
Config/testthat/edition: 3
LinkingTo: Rcpp, RcppArmadillo, RcppParallel
biocViews: DifferentialExpression, Microbiome, Normalization,
        Sequencing, Metagenomics, Software, MultipleComparison
Depends: R (>= 4.1.0)
LazyData: false
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/ADAPT
git_branch: RELEASE_3_23
git_last_commit: 6e0cbf8
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-28 20:32:32 UTC; biocbuild
Author: Mukai Wang [aut, cre] (ORCID: <https://orcid.org/0000-0002-1413-1904>),
  Simon Fontaine [ctb],
  Hui Jiang [ctb],
  Gen Li [aut, ctb]
Maintainer: Mukai Wang <wangmk@umich.edu>

# Generated by roxygen2: do not edit by hand

export(adapt)
exportClasses(DAresult)
exportMethods(plot)
exportMethods(summary)
import(RcppArmadillo)
importFrom(Rcpp,sourceCpp)
importFrom(RcppParallel,RcppParallelLibs)
importFrom(ggplot2,.data)
importFrom(ggplot2,aes)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(ggrepel,geom_label_repel)
importFrom(methods,setClass)
importFrom(phyloseq,filter_taxa)
importFrom(phyloseq,nsamples)
importFrom(phyloseq,otu_table)
importFrom(phyloseq,prune_samples)
importFrom(phyloseq,sample_data)
importFrom(phyloseq,sample_sums)
importFrom(phyloseq,tax_table)
importFrom(phyloseq,taxa_are_rows)
importFrom(stats,median)
importFrom(stats,model.matrix)
importFrom(stats,optim)
importFrom(stats,p.adjust)
importFrom(stats,pchisq)
importFrom(stats,predict)
importFrom(stats,qchisq)
importFrom(stats,smooth.spline)
useDynLib(ADAPT)

# ADAPT

## Installation

There are two ways of installing the package. One is through the GitHub repository and the other is through the Bioconductor repository.

```{r, eval=FALSE}
# install Bioconductor first
if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

# the first way is to install from GitHub
if (!require("ADAPT"))
    BiocManager::install("mkbwang/ADAPT", build_vignettes = TRUE)

# the second way is install from Bioconductor repository
if (!require("ADAPT"))
    BiocManager::install("ADAPT", version="devel", build_vignettes = TRUE)
```
The installation may take around five minutes.

_The authors are preparing to submit this package to bioconductor._


## Introduction

`ADAPT` stands for "Analysis of microbiome differential abundance by pooling Tobit models". There are two main innovations:

* `ADAPT` treats zero counts as left censored observations and use Tobit models to model the log count ratios.
* `ADAPT` has an innovative way of selecting non-differentially abundant taxa as reference taxa. It then uses the reference taxa to find the differentially abundant ones.

To learn more about the usage of `ADAPT`, type in R console

```{r, eval=FALSE}
library(ADAPT)
browseVignettes(package="ADAPT")
```

The codes of simulation studies and real data analyses in the manuscript are in this [GitHub repository](https://github.com/mkbwang/ADAPT_example).

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----echo=FALSE, out.width='70%', fig.align = "center", fig.cap="Workflow of ADAPT"----
knitr::include_graphics('./flowchart.png')

## ----eval=FALSE---------------------------------------------------------------
# # install through GitHub
# if(!require("ADAPT")) BiocManager::install("mkbwang/ADAPT", build_vignettes = TRUE)
# 
# # install through Bioconductor
# if(!require("ADAPT")) BiocManager::install("ADAPT", version="devel",
#                                            build_vignettes = TRUE)
# 

## ----load data----------------------------------------------------------------
library(ADAPT)
data(ecc_saliva)

## ----adapt--------------------------------------------------------------------
saliva_output_noadjust <- adapt(input_data=ecc_saliva, cond.var="CaseStatus", base.cond="Control")
saliva_output_adjust <- adapt(input_data=ecc_saliva, cond.var="CaseStatus", base.cond="Control", adj.var="Site")

## ----getsummary---------------------------------------------------------------
DAtaxa_result <- summary(saliva_output_noadjust, select="da")

## ----printtable---------------------------------------------------------------
head(DAtaxa_result)

## ----volcano, fig.width=5, fig.height=3, fig.align = "center", fig.cap="Volcano plot for differential abundance analysis between saliva samples of children who developed early childhood dental caries and those who didn't"----
plot(saliva_output_noadjust, n.label=5)

## ----sessioninfo--------------------------------------------------------------
sessionInfo()

# class and methods for ADAPT result

#' Differential abundance analysis result
#' 
#' @description
#' An S4 class to represent ADAPT analysis results
#' 
#' @details
#' The analysis result object contains the analysis name, reference taxa, DA taxa,
#' detailed analysis results as a dataframe and the input phyloseq object.
#' The analysis name contains the condition variable. The reference taxa `reference`
#' must be nonempty. DA taxa `signal` may be an empty string if no taxa are differentially abundant.
#' The `details` dataframe contains the taxa names, the prevalence of taxa, the estimated
#' log10 absolute abundance fold changes, the raw hypothesis test p-values and BH-adjusted p-values.
#' 
#' @slot DAAname The name of differential abundance analysis
#' @slot reference A vector of taxa names corresponding to all the reference taxa
#' @slot signal A vector of taxa names corresponding to all the DA taxa
#' @slot details A dataframe with the analysis results for all taxa
#' @slot input Input phyloseq object
#' @importFrom methods setClass
#' @export
setClass("DAresult",
         slots=c(
           DAAname="character",
           reference="character",
           signal="character",
           details="data.frame",
           input="phyloseq"
         ))

setValidity("DAresult", function(object) {
  if (!is.character(object@DAAname)) {
    "@DAAname must be a string"
  } else if (length(object@reference) == 0) {
    "There must be at least one reference taxa"
  } else{
    TRUE
  }
})



#' @importFrom phyloseq nsamples
setMethod("show", "DAresult", function(object){
  
  cat(sprintf("Differential abundance analysis for %s\n", object@DAAname))
  cat(sprintf("%d taxa and %d samples analyzed\n", nrow(object@details), nsamples(object@input)))
  num_DAtaxa <- length(object@signal)
  if (num_DAtaxa == 1 & object@signal[1] == ""){
    num_DAtaxa <- 0
  }
  cat(sprintf("%d taxa are selected as reference and %d taxa are differentially abundant\n",
              length(object@reference), num_DAtaxa))
  
})

#' Summary of differential abundance analysis 
#'
#' @description
#' Summary function for `DAresult` type object
#' 
#' @details 
#' This customized summary function reports the dimension of input count table,
#' number of reference taxa and number of differentially abundant taxa. It also returns
#' a data frame with the detailed analysis result and taxonomy of all the taxa. The
#' user can choose to only get the detailed analysis result of DA taxa or the reference taxa
#' through the `select` parameter.
#' 
#' @param object analysis result in `DAresult` type
#' @param select Taxa whose results to be returned, can be all the taxa ("all"),
#' only the differentially abundant taxa ("da") or reference taxa ("ref").
#' @returns A dataframe with detailed analysis results
#' @export
#' @importFrom phyloseq nsamples tax_table
#' @examples
#' data(ecc_saliva)
#' saliva_results <- adapt(input_data=ecc_saliva, cond.var="CaseStatus", 
#'        base.cond="Control", adj.var="Site")
#' summary(saliva_results, select="da")
setMethod("summary", "DAresult", function(object, select=c("all", "da", "ref")){
  
  select <- match.arg(select)
  # first show overall information
  show(object)
  
  # load all the taxonomical information from the input data if any
  full_taxonomy_information = tryCatch({
    as.data.frame(tax_table(object@input))
  }, warning = function(w){
    NULL
  }, error = function(e){
    NULL
  }, finally = function(f){
    NULL
  })
  
  taxonomy_information <- NULL
  subset_result <- NULL
  combined_result <- NULL
  
  # load the full analysis result
  full_result <- object@details
  DAtaxa <- object@signal
  reftaxa <- object@reference
  if (select == "da" & length(DAtaxa) == 1 & DAtaxa[1] == ""){
    combined_result <- NULL
  } else if(select == "da"){
    subset_result <- full_result[DAtaxa, , drop=FALSE]
    if (!is.null(full_taxonomy_information)) {
      taxonomy_information <- full_taxonomy_information[DAtaxa, , drop=FALSE]
    }
  } else if(select == "ref") {
    subset_result <- full_result[reftaxa, ,drop=FALSE]
    if (!is.null(full_taxonomy_information)) {
      taxonomy_information <- full_taxonomy_information[reftaxa, , drop=FALSE]
    }
  } else{ # select all the analyzed taxa by default
    subset_result <- full_result
    if (!is.null(full_taxonomy_information)) {
      taxonomy_information <- full_taxonomy_information[rownames(subset_result), , drop=FALSE]
    }
  }
  
  # column combine the analysis result and taxonomy information
  if (!is.null(taxonomy_information) & !is.null(subset_result)){
    combined_result <- cbind(subset_result, taxonomy_information)
  } else if (!is.null(subset_result)){
    combined_result <- subset_result
  }
  
  invisible(combined_result)
  
})

#' Plotting differential abundance analysis results
#' 
#' @description
#' Volcano plot of ADAPT results
#' 
#' @details
#' The customized plot function for `DAresult` type object generates a volcano plot with
#' the differentially abundant taxa highlighted. The users can decide how many taxa with 
#' the smallest p-values are labeled on the plot.
#' 
#' @param x analysis result in `DAresult` type
#' @param n.label Number of taxa to label on the plot. Note that no taxa will be labeled if
#' no DA taxa.
#' @returns A ggplot object of the volcano plot 
#' @importFrom ggplot2 ggplot aes geom_point xlab ylab element_text .data
#' @importFrom ggplot2 theme_bw theme scale_color_manual geom_vline
#' @importFrom ggrepel geom_label_repel
#' @export
#' @examples
#' data(ecc_saliva)
#' saliva_results <- adapt(input_data=ecc_saliva, cond.var="CaseStatus", 
#'        base.cond="Control", adj.var="Site")
#' plot(saliva_results, n.label=10)
setMethod("plot", "DAresult", function(x, n.label=5){
  
  n.label <- as.integer(n.label)
  stopifnot(n.label >= 0)
  
  model_details <- x@details
  ## just in case some taxa are too rare to be estimated
  model_details <- model_details[!is.na(model_details$teststat), ]
  ## calculate negative log10 p value
  model_details$neglog10pval <- -log10(model_details$pval)
  title_x <- sprintf("Log10 Fold Change for %s", x@DAAname)
  
  generated_plot <- NULL
  # first generate basic plot
  if(x@signal[1] != ""){ # has DA taxa
    model_details$isDA <- FALSE
    model_details[x@signal, "isDA"] <- TRUE
    
    if(n.label > 0){
      n.label <- min(n.label, length(x@signal))
      index_withlabels <- which(rank(model_details$pval) <= n.label)
      model_details$label <- ""
      model_details$label[index_withlabels] <- model_details$Taxa[index_withlabels]
    }
    
    generated_plot <- ggplot(model_details, aes(x=.data$log10foldchange, y=.data$neglog10pval))+
      geom_point(alpha=0.8, aes(color=.data$isDA)) +
      xlab(title_x) + ylab("-Log10 p-value") + theme_bw() + 
      theme(legend.position="none", axis.title=element_text(size=10), 
            axis.text=element_text(size=10)) + 
      scale_color_manual(values=c("#616161", "#ff0066")) +
      geom_vline(xintercept=0, linetype="dashed", color = "blue")
    
    if (n.label > 0){ # add labels
      generated_plot <- generated_plot + 
        geom_label_repel(aes(label = .data$label),
                         size=2,
                         max.overlaps = 20,
                         box.padding   = 0.35,
                         point.padding = 0.5,
                         segment.color = 'grey50')
    }
    
  } else{ # no DA taxa
    generated_plot <- ggplot(model_details, aes(x=.data$log10foldchange, y=.data$neglog10pval)) + 
      geom_point(alpha=0.8, color="#616161") + 
      xlab(title_x) + ylab("-Log10 p-value") + theme_bw() + 
      theme(axis.title=element_text(size=10), axis.text=element_text(size=10)) + 
      geom_vline(xintercept=0, linetype="dashed", color = "blue")
  }
  
  return(generated_plot)
  
})

#' @useDynLib ADAPT
#' @importFrom stats pchisq
#' @importFrom stats smooth.spline
#' @importFrom stats predict
#' @importFrom Rcpp sourceCpp
#' @import RcppArmadillo
#' @importFrom RcppParallel RcppParallelLibs
count_ratio <- function(count_table, design_matrix, reftaxa=NULL, censor=1, test_all=FALSE){
  alltaxa <- colnames(count_table)
  if (is.null(reftaxa)) reftaxa <- alltaxa
  TBDtaxa <- NULL
  if (test_all){ # examine count ratio between other taxa and the subset
    TBDtaxa <- alltaxa
  } else{ # relative abundance within the subset of taxa
    TBDtaxa <- reftaxa
  }

  refcounts <- rowSums(count_table[, reftaxa]) # sum of reference taxa as denominator
  # remove the samples with zero counts in the reference set
  null_filter <- refcounts != 0
  refcounts <- refcounts[null_filter] # denominator
  subset_designmatrix <- design_matrix[null_filter, ]
  TBD_counts <- count_table[null_filter, TBDtaxa, drop=FALSE]
  existence <- 1*(TBD_counts > 0) # indicator matrices of zero counts
  TBD_counts[TBD_counts == 0] <- censor
  prevalences <- colMeans(existence)
  # metadata <- metadata[null_filter, ,drop=FALSE]
  # design_mat <- cbind(1, metadata[, c(covar, adjust)]) |> as.matrix()


  CR_result <- data.frame(Taxa = TBDtaxa,
                          prevalence=prevalences,
                          log10foldchange=0,
                               teststat=0,
                               pval=0)

  estimation_result <- cr_estim(count_mat=TBD_counts, refcounts=refcounts, 
                                Delta=existence, X=subset_designmatrix)
  CR_result$log10foldchange <- estimation_result[, 1] / log(10)
  CR_result$teststat <- estimation_result[, 2]
  CR_result$log10foldchange[estimation_result[, 3] == 1] <- NA # some taxa may be too rare for statistical inference
  CR_result$teststat[estimation_result[, 3] == 1] <- NA # some taxa may be too rare for statistical inference
  CR_result$pval <- 1 - pchisq(CR_result$teststat, 1)

  return(CR_result)
}

#' Saliva samples from early childhood dental caries studies
#' 
#' A phyloseq object with 161 samples and 280 ASVs for 16S sequencing of saliva 
#' samples. The samples were collected from 
#' 12-month-old infants. 84 out of 161 children developed dental caries after 
#' 36 months old. All samples have been de-identified.
#' 
#' @usage data(ecc_saliva)
#' 
#' 
#' @format 
#' The metadata of `ecc_saliva` has two columns
#' \describe{
#'   \item{CaseStatus}{Whether the child developed dental caries after 36 months old}
#'   \item{Site}{The location of sample collection (Site 1 or Site 2).}
#' }
#' @source The original publication of this data is "Evaluating the ecological 
#' hypothesis: early life salivary microbiome assembly predicts dental caries 
#' in a longitudinal case-control study" (Blostein etal, 2022). 
#' The sequence data are available under Project number PRJNA752888.
"ecc_saliva"




#' Plaque samples from early childhood dental caries studies
#' 
#'
#' A phyloseq object with 30 samples and 610 taxa for whole genome sequencing of plaque samples. 
#' The samples were collected from children at 36, 48 or 60 months old.
#' 15 samples were from teeth with dental lesions and 15 samples were controls. 
#' The samples of cases were collected at the onset visit.
#'
#' @usage data(ecc_plaque)
#'
#' @format 
#' The metadata of `ecc_plaque` has two columns
#' \describe{
#'   \item{CaseStatus}{Whether the kids had dental caries}
#'   \item{Site}{The location of sample collection (Site 1 or Site 2).}
#' }
#' @source The original publication of this data is "Evaluating the ecological 
#' hypothesis: early life salivary microbiome assembly predicts dental caries 
#' in a longitudinal case-control study" (Blostein etal, 2022). 
#' The sequence data are available under Project number PRJNA752888.
"ecc_plaque"

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

cr_estim <- function(count_mat, refcounts, Delta, X) {
    .Call('_ADAPT_cr_estim', PACKAGE = 'ADAPT', count_mat, refcounts, Delta, X)
}

# inspired by ClassComparison package

expit <- function(a) { exp(a)/(1+exp(a)) }

# log likelihood of mixture with parameters a and L
BUM_llk <- function(vec, pvals) {
  psi <- vec[1]
  phi <- vec[2]
  a <- expit(psi)
  L <- expit(phi)
  sum(log(L+(1-L)*a*pvals^(a-1)))
}

BUM_fit <- function(pvals, ...) {
  if (all(is.na(pvals))) {
    stop("all p-values were NA; nothing to compuite")
  }
  orig.pvals <- pvals
  if (any(is.na(pvals))) {
    pvals <- pvals[!is.na(pvals)]
  }
  if (any(pvals < 0) || any(pvals > 1)) {
    stop("all p-values must be between 0 and 1")
  }
  if(min(pvals)==0) {
    min.nonzero <- min(pvals[pvals>0])
    pvals[pvals==0] <- min.nonzero/2
  }
  fitted_values <- optim(c(1/2, 1/2), BUM_llk, ...,
                    method='BFGS', pvals=pvals, control=list(fnscale=-1))	# least squares fit
  psi <- fitted_values$par[1]
  phi <- fitted_values$par[2]
  ahat <- expit(psi)	# MLE estimate of a
  lhat <- expit(phi)	# MLE estimate of L
  pihat <- lhat + (1-lhat)*ahat	# upper bound on percent unchanged
  output <- list(estim_params = fitted_values$par,
                 pihat=pihat,
                 pvals=orig.pvals)
  return(output)
}

#' @importFrom phyloseq filter_taxa prune_samples otu_table sample_data taxa_are_rows sample_sums
#' @importFrom stats model.matrix
preprocess <- function(input_data, cond.var, base.cond, adj.var, prev.filter, depth.filter){
  
  # check if input data type is phyloseq
  stopifnot("Input data isn't a phyloseq object!" = is(input_data, 'phyloseq'))
  # check the prevalence threshold and the sequencing depth threshold
  stopifnot("The prevalence filter (prev.filter) has to be a positive number between 0 and 1!" = prev.filter >=0 & prev.filter < 1)
  stopifnot("The sequencing depth filter (depth.filter) has to be a nonnegative number!" = depth.filter >= 0)
  # filter phyloseq object based on taxa prevalence and sequencing depth
  subset_data <- filter_taxa(input_data, function(x) mean(x>0) > prev.filter, TRUE)
  subset_data <- prune_samples(sample_sums(subset_data) > depth.filter, subset_data)
  
  # check if the variables exist in the metadata
  metadata <- data.frame(sample_data(subset_data))
  allcols <- colnames(metadata)
  stopifnot("The main variable name for conditions need to be a string." = 
              is(cond.var, "character"))

  stopifnot("The variables for adjustments should be either NULL or a vector of character strings." = 
              is(adj.var, "character") | is(adj.var, "NULL"))

  selected_cols <- c(cond.var, adj.var)
  if (!all(selected_cols %in% allcols)){
    unavailable_cols <- selected_cols[!selected_cols %in% allcols]
    stop(sprintf("Some columns are not available in the metadata! (%s)", 
                 paste(unavailable_cols, collapse=",")))
  }
  subset_metadata <- metadata[, c(cond.var, adj.var), drop=FALSE]
  # dichotomize categorical variables if selected, set up design matrix
  if (any(is.na(subset_metadata))){
    stop("No missing data allowed in the metadata!")
  }
  main_variable <- subset_metadata[, cond.var]
  if (length(unique(main_variable)) == 1){
    stop("All samples share the same condition!")
  }
  if (!is.numeric(main_variable)){
    if (is.null(base.cond)){
      base.cond <- unique(main_variable)[1]
    } else if (!base.cond %in% main_variable) {
      base.cond <- unique(main_variable)[1]
    }
    cat(sprintf("Choose '%s' as the baseline condition\n", base.cond))
    if (length(unique(main_variable)) == 2){
      others <- setdiff(main_variable, base.cond)
    } else{
      others <- "others"
    }
    main_variable <- as.integer(main_variable != base.cond)
    cond.var <- sprintf("%s (%s VS %s)", cond.var, others, base.cond)
  }
  adjustments <- NULL
  if (!is.null(adj.var)){
    adjustments <- subset_metadata[, adj.var, drop=FALSE]
    adjustments<- model.matrix(~., data=adjustments)
    adjustments<- adjustments[, -1] # remove intercept
  }
  complete_design_matrix <- cbind(1, main_variable, adjustments)
  
  # parse the count matrix and edit the count_ratio function
  count_table <- otu_table(subset_data)
  if (taxa_are_rows(subset_data)){
    count_table <- t(count_table)
  }
  
  if (any(is.na(count_table))){
    stop("No missing data allowed in the count table!")
  }
  
  cat(sprintf("%d taxa and %d samples being analyzed...\n", 
              ncol(count_table), nrow(count_table)))
  
  output <- list(count_table=count_table, design_matrix=complete_design_matrix,
                 DAAname = cond.var)
  
  return(output)
  
}

#' ADAPT
#' 
#' @description 
#' Analysis of microbiome differential abundance by pooling tobit models
#' 
#' @details
#' ADAPT takes in a metagenomics count table as a phyloseq object. 
#' The phyloseq object needs to have metadata containing 
#' at least one variable `cond.var` representing the conditions that the user is testing on. 
#' The condition variable `cond.var` can be numeric (as a continuous variable) or character (representing categorical variable).
#' ADAPT does not support multigroup comparison yet. If there are multiple conditions,
#' the user can specify the condition to single out through `base.cond`. ADAPT then carry out DAA between
#' the selected `base.cond` and all the others. ADAPT allows adjusting for other covariates. The 
#' user can specify all the covariates to adjust for by specifying `adj.var` with a vector of variable names.
#'
#' Differential abundance analysis may be too challenging for rare taxa and samples with too low sequencing depth.
#' The users can filter out taxa whose prevalences are lower than `prev.filter` (default 0.05). The users 
#' can also filter out samples whose sequencing depths (library sizes) are smaller than `depth.filter` (default 1000).
#' 
#' One major feature of ADAPT is treating zero counts as left censored observations and use Tobit models for log count ratios.
#' The zero counts by default are left censored at one. The users can change the value to censor at through `censor`.
#' Change the cutoff of BH-adjusted p-values with `alpha` (default 0.05) for calling DA taxa.
#' 
#' The returned value of `adapt` is a customized S4 type called `DAresult`. We have developed two helper functions `summary`
#' and `plot` for this special data type.
#' 
#' 
#' @useDynLib ADAPT
#' @param input_data a phyloseq object
#' @param cond.var the variable representing the conditions to compare, a character string
#' @param base.cond the condition chosen as baseline. This is only used when the condition is categorical.
#' @param adj.var the names of the variables to be adjusted, a vector of character strings
#' @param censor the value to censor at for zero counts, default 1
#' @param prev.filter taxa whose prevalences are smaller than the cutoff will be excluded from analysis, default 0.05
#' @param depth.filter a sample would be discarded if its library size is smaller than the threshold
#' @param alpha the cutoff of the adjusted p values
#' @importFrom stats optim median p.adjust qchisq
#' @returns a `DAresult` type object contains the input and the output. Use summary and plot to explore the output
#' @export
#' 
#' @examples
#' data(ecc_plaque)
#' plaque_results <- adapt(input_data=ecc_plaque, cond.var="CaseStatus", 
#'        base.cond="case")
#' data(ecc_saliva)
#' saliva_results <- adapt(input_data=ecc_saliva, cond.var="CaseStatus", 
#'        base.cond="Control", adj.var="Site")
adapt <- function(input_data, cond.var, base.cond = NULL, adj.var=NULL, censor=1,
                 prev.filter=0.05, depth.filter=1000, alpha=0.05){
  
  # preprocess input phyloseq object
  preprocessed_output <- preprocess(input_data, cond.var, base.cond, adj.var, prev.filter, depth.filter)
  
  # check if the arguments are valid
  stopifnot("The zero counts need to be censored at a positive number!" = censor > 0)
  stopifnot("The cutoff for adjusted p-values needs to be between 0 and 0.5!" = alpha > 0 & alpha < 0.5)
  
  count_table <- preprocessed_output$count_table
  complete_design_matrix <- preprocessed_output$design_matrix
  DAAname <- preprocessed_output$DAAname
  
  taxa_names <- colnames(count_table)

  
  reftaxa <- taxa_names # initially all the taxa are reference taxa(relative abundance)
  cat("Selecting Reference Set... ")
  while(1){
    relabd_result <- count_ratio(count_table = count_table, design_matrix = complete_design_matrix,
                                  censor = censor, reftaxa = reftaxa, test_all=FALSE)
    estimated_effect <- relabd_result$log10foldchange
    pvals <- relabd_result$pval
    names(pvals) <- relabd_result$Taxa
    names(estimated_effect) <- relabd_result$Taxa
    # check distribution of p values
    bumfit <- BUM_fit(pvals)
    loglik <- BUM_llk(bumfit$estim_params, pvals[!is.na(pvals)])
    if (2*loglik > qchisq(0.95, 1)){ # need to continue shrinking reference taxa set
      distance2med <- abs(estimated_effect - median(estimated_effect, na.rm=TRUE))
      sorted_distance <- sort(distance2med)
      ordered_taxanames <- names(sorted_distance)
      reftaxa <- ordered_taxanames[seq_len(length(ordered_taxanames)/2)]
    } else{
      break
    }
  }
  cat(sprintf("%d taxa selected as reference\n", length(reftaxa)))
  all_CR_results <- count_ratio(count_table=count_table, design_matrix=complete_design_matrix,
                                censor = censor, reftaxa=reftaxa, test_all=TRUE)

  all_pvals <- all_CR_results$pval
  names(all_pvals) <- all_CR_results$Taxa
  all_adjusted_pvals <- p.adjust(all_pvals, method="BH")
  all_CR_results$adjusted_pval <- all_adjusted_pvals

  significant_pvals <- all_adjusted_pvals[all_adjusted_pvals < alpha & !is.na(all_adjusted_pvals)]
  if (length(significant_pvals) > 0){
    DiffTaxa <- names(significant_pvals)
  } else{
    DiffTaxa <- c()
  }
  cat(sprintf("%d differentially abundant taxa detected\n", length(DiffTaxa)))
  if(is.null(DiffTaxa)) DiffTaxa <- ""
  output <- new("DAresult",
                DAAname=DAAname,
                reference=reftaxa, 
                signal=DiffTaxa,
                details=all_CR_results,
                input=input_data)

  invisible(output)
}

test_that("balanced change", {
  phyobj_balanced <- balanced_case$phyobj
  balanced_truth <- balanced_case$truth
  balanced_output <- adapt(input_data=phyobj_balanced, 
                           cond.var="main", adj.var=c("confounder"))
  balanced_results <- summary(balanced_output, 'da')
  signals <- balanced_results$Taxa
  true_signals <- balanced_truth$taxaname[balanced_truth$isDA]
  FDR_balanced <- 1-mean(signals %in% true_signals)
  expect_lt(FDR_balanced, 0.05)
})


test_that("unbalanced change", {
  phyobj_unbalanced <- unbalanced_case$phyobj
  unbalanced_truth <- unbalanced_case$truth
  unbalanced_output <- adapt(input_data=phyobj_unbalanced, 
                             cond.var="main", adj.var="confounder")
  unbalanced_result <- summary(unbalanced_output, 'da')
  signals <- unbalanced_result$Taxa
  true_signals <- unbalanced_truth$taxaname[unbalanced_truth$isDA]
  FDR_unbalanced <- 1-mean(signals %in% true_signals)
  expect_lt(FDR_unbalanced, 0.05)
})

test_that("null case", {
  null_output <- adapt(input_data=phyobj_null, 
                       cond.var="main", adj.var="confounder")
  null_results <- summary(null_output)
  FPR <- mean(null_results$pval < 0.05)
  expect_lt(FPR, 0.05)
})

#!/bin/bash
TOP=$(cd $(dirname $0) && pwd)

function doc() {
  local MODULE=$1
  local OUTPUT_DIR=${TOP}/build/doc/html/$(dirname $MODULE)
  PYTHONPATH=. pdoc --html\
       --overwrite\
       --html-dir ${OUTPUT_DIR}\
       $MODULE
}
cd ${TOP}
doc adapt
for i in $(find adapt/*| grep -v pyc); do
  doc $i
done
for i in $(find adapt/* -type d | sed -s 's/\//./g'); do
  doc $i
done

#! /bin/bash

ADAPT_DIR=$(dirname $0)

do_lint () {
    flake8 "${ADAPT_DIR}/adapt" --select=E9,F63,F7,F82 --show-source && \
    flake8 "${ADAPT_DIR}/test" --select=E9,F63,F7,F82 --show-source
}

do_test () {
    pytest "${ADAPT_DIR}/test/"*
}

show_help () {
    echo "Tests for adapt."
    echo "If no arguments are given, both test and linting is performed."
    echo "Otherwise the argument will determine which part is performed."
    echo ""
    echo "    Usage: $0 [test/lint]"
    echo ""
    echo "Arguments:"
    echo "  test: Only run the tests."
    echo "  lint: Only perform codestyle and static analysis."
}

if [[ $# == 0 ]]; then
    do_lint || exit $? # Exit on failure
    do_test || exit $? # Exit on failure
elif [[ $# == 1 ]]; then
    if [[ $1 == "lint" ]]; then
        do_lint
    elif [[ $1 == "test" ]]; then
        do_test
    else
        show_help
    fi
else
    show_help
fi

# Contributing

Pull requests are an important part of improving Adapt and making it easier for people to adopt. We\'re 
constantly striving to better the code and the documentation, and gladly welcome input from the community.
In order to keep the quality of Adapt high, we have a few guidelines for changes.

## Getting Started
 * Set up your environment
As described in our README, set up your development environment. We use virtualenv and virtualenvwrapper, tools that are available for all platforms.

To ensure your environment is set up properly, install requirements.txt as well as test-requirements.txt, then attempt to run the existing unit tests with the following command:

```
PYTHONPATH=. python run_tests.py
```

 * Submit a ticket for your issue, assuming one doesn't already exist.
 * Fork the repository on GitHub

## Making changes
 * Create a topic branch off of master
 * Make commits of logical units.
 * Rebase off master before submitting.
 * Make sure you have added the necessary unit tests for your changes.
 * Run _all_ the tests to assure nothing else was accidentally broken.

## Submitting Changes
 * Push your changes to a topic branch in your fork of the repository.
 * Submit a pull request to the MycroftAI/adapt repository.
 * The MycroftAI organization reviews pull requests regularly. We will provide feedback as necessary, and TravisCI will verify that your changes pass all tests.
 * After feedback, we expect responses in 2 weeks. After 2 weeks, we may close or coopt the change request.


#### Modeled after the Puppet Labs contributing documentation.

# Adapt Intent Determination
The Adapt Intent Parser is a flexible and extensible intent definition and determination framework. It is intended to parse natural language text into a structured intent that can then be invoked programmatically. It uses a technique known as known-entity tagging to tag pre-defined vocabulary within utterances. These tags can then be passed to an intent definition for validation. 

## Getting Started
The Parser depends on a tokenizer (splits an utterance into tokens), and a prefix tree loaded with vocabulary. 

        # Instantiate a Trie for in-memory index, a tokenizer, and the entity tagger.
        trie = Trie()
        tokenizer = EnglishTokenizer()
        tagger = EntityTagger(self.trie, self.tokenizer)
        # load vocabulary into index
        trie.insert("play", "PlayVerb")
        trie.insert("the big bang theory", "Television Show")
        trie.insert("the big", "Not a Thing")
        trie.insert("barenaked ladies", "Radio Station")
        # Instantiate parser
        parser = Parser(self.tokenizer, self.tagger)

        # Define intent
        intent = IntentBuilder("play television intent")\
            .require("PlayVerb")\
            .require("Television Show")\
            .build()

        # parser.parse returns a generator, which returns (using a greedy algo) highest confidence parse first, for short-circuiting
        for result in self.parser.parse("play the big bang theory"):
            # validate an intent. A score of 0.0 indicates the parse result is not a valid representation of the intent.
            result_intent = intent.validate(result.get('tags'), result.get('confidence'))
            assert result_intent.get('confidence') > 0.0
            assert result_intent.get('PlayVerb') == 'play'
            assert result_intent.get('Television Show') == "the big bang theory"

---
title: "ADAPT Tutorial"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{ADAPT Tutorial}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
bibliography: references.bib
csl: nature-methods.csl
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```


# Introduction

This is the tutorial for `ADAPT` (analysis of microbiome differential abundance analysis by pooling Tobit models). Differential abundance analysis (DAA) is one of the fundamental steps in microbiome research [@LiReview]. The metagenomics count data have excessive zeros and are compositional, causing challenges for detecting microbiome taxa whose abundances are associated with different conditions. `ADAPT` overcomes these challenges differently from other DAA methods. First, `ADAPT` treats zero counts as left censored observations and apply Tobit models which are censored regressions [@tobit] for modeling the log count ratios. Second, `ADAPT` has an innovative and theoretically justified way of finding non-differentially abundant taxa as reference taxa. It then applies Tobit models to log count ratios between individual taxa and the summed counts of reference taxa to identify differentially abundant taxa. The overall workflow is presented in the graph below.

```{r echo=FALSE, out.width='70%', fig.align = "center", fig.cap="Workflow of ADAPT"}
knitr::include_graphics('./flowchart.png')
```

There are two ways to install the package, either through GitHub or Bioconductor.

```{r, eval=FALSE}
# install through GitHub
if(!require("ADAPT")) BiocManager::install("mkbwang/ADAPT", build_vignettes = TRUE)

# install through Bioconductor
if(!require("ADAPT")) BiocManager::install("ADAPT", version="devel", 
                                           build_vignettes = TRUE)

```

# Main function `adapt`

The main function `adapt` takes in a phyloseq [@phyloseq] object that needs to have at least a count table and a sample metadata. The metadata must contain one variable that represents the condition to test on. We currently allow binary and continuous conditions. If there are more than two groups for the condition variable, the user may single out one group to compare with all the others for DAA. `ADAPT` allows adjusting for extra covariates. There are eight arguments in the `adapt` function: 

1. `input_data` is the phyloseq object with the count matrix and metadata. This argument is required.
2. `cond.var` is the variable in the metadata that represents the condition to test on. This argument is required.
3. `base.cond` is the group that the user is interested in comparing against other groups. This argument is only needed if the condition is categorical.
4. `adj.var` contains the covariates to adjust for. It can be empty or be a vector of variable names.
5. `censor` is the value to censor at for all the zero counts. By default the zeros are censored at one.
6. `prev.filter` is the threshold for filtering out rare taxa. By default taxa with prevalence lower than 0.05 are filtered out before analysis.
7. `depth.filter` is the threshold for filtering out samples with low sequencing depths. The default threshold is 1000 reads.
8. `alpha` is the cutoff for the Benjamini-hochberg adjusted p-values to decide differentially abundant taxa. The default is 0.05.

The `ADAPT` package contains two metagenomics datasets from an early childhood dental caries study [@ECCpaper1]. One corresponds to 16S rRNA sequencing of 161 saliva samples collected from 12-month-old infants (`ecc_saliva`). The other corresponds to shotgun metagenomic sequencing of 30 plaque samples collected from kids between 36 and 60 months old (`ecc_plaque`). In this vignette let's use the saliva data to find out differentially abundant taxa between kids who developed early childhood caries (ECC) after 36 months old and those who didn't. Besides the main variable `CaseStatus`, there is another variable `Site` representing the site where each sample was collected. Both variables are discrete and contain two categories each.  

```{r load data}
library(ADAPT)
data(ecc_saliva)
```

We can run `adapt` with or without covariate adjustment. The returned object is a customized S4-type object with slots corresponding to analysis name, reference taxa, differentially abundant taxa, detailed analysis results (a dataframe) and input phyloseq object.


```{r adapt}
saliva_output_noadjust <- adapt(input_data=ecc_saliva, cond.var="CaseStatus", base.cond="Control")
saliva_output_adjust <- adapt(input_data=ecc_saliva, cond.var="CaseStatus", base.cond="Control", adj.var="Site")
```


The `Site` is not a confounding variable, therefore the DAA results only differ by one taxon. 

# Explore Analysis Results

We have developed two utility functions for result exploration. `summary` returns a dataframe with the estimated log10 abundance fold changes, hypothesis test p-values and taxonomy (if provided in the phyloseq object). The user can choose to return results of all the taxa, the differentially abundant taxa only or the reference taxa only through the `select` variable ("all", "da" or "ref"). We choose to return the results of only the DA taxa below.

```{r getsummary}
DAtaxa_result <- summary(saliva_output_noadjust, select="da")
```

```{r printtable}
head(DAtaxa_result)
```


We have also developed a `plot` function to generate a volcano plot. The DA taxa are highlighted in red. The user can decide how many points with the smallest p-values are labeled (default 5) with `n.label` argument.


```{r volcano, fig.width=5, fig.height=3, fig.align = "center", fig.cap="Volcano plot for differential abundance analysis between saliva samples of children who developed early childhood dental caries and those who didn't"}
plot(saliva_output_noadjust, n.label=5)
```

# Session Information and References
```{r sessioninfo}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{ecc_plaque}
\alias{ecc_plaque}
\title{Plaque samples from early childhood dental caries studies}
\format{
The metadata of \code{ecc_plaque} has two columns
\describe{
\item{CaseStatus}{Whether the kids had dental caries}
\item{Site}{The location of sample collection (Site 1 or Site 2).}
}
}
\source{
The original publication of this data is "Evaluating the ecological
hypothesis: early life salivary microbiome assembly predicts dental caries
in a longitudinal case-control study" (Blostein etal, 2022).
The sequence data are available under Project number PRJNA752888.
}
\usage{
data(ecc_plaque)
}
\description{
A phyloseq object with 30 samples and 610 taxa for whole genome sequencing of plaque samples.
The samples were collected from children at 36, 48 or 60 months old.
15 samples were from teeth with dental lesions and 15 samples were controls.
The samples of cases were collected at the onset visit.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\docType{class}
\name{DAresult-class}
\alias{DAresult-class}
\title{Differential abundance analysis result}
\description{
An S4 class to represent ADAPT analysis results
}
\details{
The analysis result object contains the analysis name, reference taxa, DA taxa,
detailed analysis results as a dataframe and the input phyloseq object.
The analysis name contains the condition variable. The reference taxa \code{reference}
must be nonempty. DA taxa \code{signal} may be an empty string if no taxa are differentially abundant.
The \code{details} dataframe contains the taxa names, the prevalence of taxa, the estimated
log10 absolute abundance fold changes, the raw hypothesis test p-values and BH-adjusted p-values.
}
\section{Slots}{

\describe{
\item{\code{DAAname}}{The name of differential abundance analysis}

\item{\code{reference}}{A vector of taxa names corresponding to all the reference taxa}

\item{\code{signal}}{A vector of taxa names corresponding to all the DA taxa}

\item{\code{details}}{A dataframe with the analysis results for all taxa}

\item{\code{input}}{Input phyloseq object}
}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{summary,DAresult-method}
\alias{summary,DAresult-method}
\title{Summary of differential abundance analysis}
\usage{
\S4method{summary}{DAresult}(object, select = c("all", "da", "ref"))
}
\arguments{
\item{object}{analysis result in \code{DAresult} type}

\item{select}{Taxa whose results to be returned, can be all the taxa ("all"),
only the differentially abundant taxa ("da") or reference taxa ("ref").}
}
\value{
A dataframe with detailed analysis results
}
\description{
Summary function for \code{DAresult} type object
}
\details{
This customized summary function reports the dimension of input count table,
number of reference taxa and number of differentially abundant taxa. It also returns
a data frame with the detailed analysis result and taxonomy of all the taxa. The
user can choose to only get the detailed analysis result of DA taxa or the reference taxa
through the \code{select} parameter.
}
\examples{
data(ecc_saliva)
saliva_results <- adapt(input_data=ecc_saliva, cond.var="CaseStatus", 
       base.cond="Control", adj.var="Site")
summary(saliva_results, select="da")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{plot,DAresult,ANY-method}
\alias{plot,DAresult,ANY-method}
\title{Plotting differential abundance analysis results}
\usage{
\S4method{plot}{DAresult,ANY}(x, n.label = 5)
}
\arguments{
\item{x}{analysis result in \code{DAresult} type}

\item{n.label}{Number of taxa to label on the plot. Note that no taxa will be labeled if
no DA taxa.}
}
\value{
A ggplot object of the volcano plot
}
\description{
Volcano plot of ADAPT results
}
\details{
The customized plot function for \code{DAresult} type object generates a volcano plot with
the differentially abundant taxa highlighted. The users can decide how many taxa with
the smallest p-values are labeled on the plot.
}
\examples{
data(ecc_saliva)
saliva_results <- adapt(input_data=ecc_saliva, cond.var="CaseStatus", 
       base.cond="Control", adj.var="Site")
plot(saliva_results, n.label=10)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{ecc_saliva}
\alias{ecc_saliva}
\title{Saliva samples from early childhood dental caries studies}
\format{
The metadata of \code{ecc_saliva} has two columns
\describe{
\item{CaseStatus}{Whether the child developed dental caries after 36 months old}
\item{Site}{The location of sample collection (Site 1 or Site 2).}
}
}
\source{
The original publication of this data is "Evaluating the ecological
hypothesis: early life salivary microbiome assembly predicts dental caries
in a longitudinal case-control study" (Blostein etal, 2022).
The sequence data are available under Project number PRJNA752888.
}
\usage{
data(ecc_saliva)
}
\description{
A phyloseq object with 161 samples and 280 ASVs for 16S sequencing of saliva
samples. The samples were collected from
12-month-old infants. 84 out of 161 children developed dental caries after
36 months old. All samples have been de-identified.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ADAPT.R
\name{adapt}
\alias{adapt}
\title{ADAPT}
\usage{
adapt(
  input_data,
  cond.var,
  base.cond = NULL,
  adj.var = NULL,
  censor = 1,
  prev.filter = 0.05,
  depth.filter = 1000,
  alpha = 0.05
)
}
\arguments{
\item{input_data}{a phyloseq object}

\item{cond.var}{the variable representing the conditions to compare, a character string}

\item{base.cond}{the condition chosen as baseline. This is only used when the condition is categorical.}

\item{adj.var}{the names of the variables to be adjusted, a vector of character strings}

\item{censor}{the value to censor at for zero counts, default 1}

\item{prev.filter}{taxa whose prevalences are smaller than the cutoff will be excluded from analysis, default 0.05}

\item{depth.filter}{a sample would be discarded if its library size is smaller than the threshold}

\item{alpha}{the cutoff of the adjusted p values}
}
\value{
a \code{DAresult} type object contains the input and the output. Use summary and plot to explore the output
}
\description{
Analysis of microbiome differential abundance by pooling tobit models
}
\details{
ADAPT takes in a metagenomics count table as a phyloseq object.
The phyloseq object needs to have metadata containing
at least one variable \code{cond.var} representing the conditions that the user is testing on.
The condition variable \code{cond.var} can be numeric (as a continuous variable) or character (representing categorical variable).
ADAPT does not support multigroup comparison yet. If there are multiple conditions,
the user can specify the condition to single out through \code{base.cond}. ADAPT then carry out DAA between
the selected \code{base.cond} and all the others. ADAPT allows adjusting for other covariates. The
user can specify all the covariates to adjust for by specifying \code{adj.var} with a vector of variable names.

Differential abundance analysis may be too challenging for rare taxa and samples with too low sequencing depth.
The users can filter out taxa whose prevalences are lower than \code{prev.filter} (default 0.05). The users
can also filter out samples whose sequencing depths (library sizes) are smaller than \code{depth.filter} (default 1000).

One major feature of ADAPT is treating zero counts as left censored observations and use Tobit models for log count ratios.
The zero counts by default are left censored at one. The users can change the value to censor at through \code{censor}.
Change the cutoff of BH-adjusted p-values with \code{alpha} (default 0.05) for calling DA taxa.

The returned value of \code{adapt} is a customized S4 type called \code{DAresult}. We have developed two helper functions \code{summary}
and \code{plot} for this special data type.
}
\examples{
data(ecc_plaque)
plaque_results <- adapt(input_data=ecc_plaque, cond.var="CaseStatus", 
       base.cond="case")
data(ecc_saliva)
saliva_results <- adapt(input_data=ecc_saliva, cond.var="CaseStatus", 
       base.cond="Control", adj.var="Site")
}
